/**
 * 
 */
package com.jujunie.service;

import java.io.Serializable;

import com.jujunie.service.log.Log;

/**
 * Value Object base implementation
 * @author julien
 * @since 1.0
 */
public abstract class VO implements Serializable {
    
    /** Timestamp to handle caching */
    private long timestamp = -1l;
    
    /**
     * Notifies that the VO has been updated, for timestamp and cache management
     * @see #isValid()
     * @see #getCachePeriod()
     */
    protected final void udatedNotification() {
        this.timestamp =  System.currentTimeMillis();
    }
    
    /**
     * Returns the cache period, in milliseconds. This is used, compared to last update timestamp, to know if the object is still
     * valid. By default, this methods returns a negative number, meanning that there is no cache period. This should
     * be overwriten if a cache period have to be used.
     * @return cache period: negative number
     * @see #isValid()
     * @see #udatedNotification()
     */
    protected long getCachePeriod() {
        return -1l;
    }
    
    /**
     * Randomly generates a cached period between provided min and max
     * @param min min
     * @param max max > min
     * @return generated cache period
     */
    protected long generateCachePeriod(long min, long max) {
        assert min < max : "Provided min must stricly inferior to max";
        double ratio = ((double) max - (double) min) * Math.random();
        long res  = Math.round((double) min + ratio);
        Log.getInstance(this).enter("Generated period between " + min + " and " + max + ": " + res);
        return res;
    }
    
    /**
     * @return true if the object is still valid
     * @see #udatedNotification()
     * @see #getCachePeriod()
     */
    public final boolean isValid() {
        if (this.timestamp == -1l ) {
            throw new IllegalStateException("No update notification received: timestamp not initialized");
        }
        long p = this.getCachePeriod();
        if (p < 0) {
            return true;
        } else {
            return System.currentTimeMillis() - this.timestamp < p;
        }
    }
    
    /**
     * Check if the given Object is null. If yes, it throws an IllegalArgumentException
     * @param name the name of the parameter, used to generate the error message
     * @param o the object to check
     * @throws IllegalArgumentException if the object is null;
     */
    protected final void setChecker(String name, Object o) throws IllegalArgumentException {
        if(o == null) {
            throw new IllegalArgumentException("The parameter " + name + " cannot be null");
        }
    }
    /**
     * Check if the given Object is null. If yes, it throws an IllegalStateException as it means the attribute have
     * never been initialized. 
     * @param name the name of the parameter, used to generate the error message
     * @param o the object to check
     * @throws IllegalStateException if the object is null;
     */
    protected final void getChecker(String name, Object o) throws IllegalStateException {
        if(o == null) {
            throw new IllegalStateException("The parameter " + name + " have not been initialized");
        }
    }
    /**
     * Check if hasBeenSet is true. If yes, it throws an IllegalStateException as it means the attribute have
     * never been initialized. 
     * @param name the name of the parameter, used to generate the error message
     * @param hasBeenSet flag
     * @throws IllegalStateException if the object is null;
     */
    protected final void getChecker(String name, boolean hasBeenSet) throws IllegalStateException {
        if(!hasBeenSet) {
            throw new IllegalStateException("The parameter " + name + " have not been initialized");
        }
    }
}
