/**
 * 
 */
package com.jujunie.service.consolidate;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;

import org.apache.commons.lang.time.DateFormatUtils;
import org.apache.commons.lang.time.DateUtils;

/**
 * Represent a selector
 * @author Julien BETI
 * @since 0.06.01
 * @version $Revision $
 */
public class ConsolidateSelector implements Serializable, Comparable<ConsolidateSelector> {

    /** serialVersionUID */
    private static final long serialVersionUID = -470407664359245298L;
    private static final String DATE_FORMAT_PATTERN = "EEE, MMM d";

    /** Type of selector */
    enum ValueType {STRING, DATE_RANGE, INT, LONG, FLOAT, DOUBLE, BOOLEAN}
    /** Date range type */
    public enum DateRangeType {NONE, DAILY, WEEKLY, MONTHLY}
    /** Type */
    private ValueType type = ValueType.STRING;
    /** String value */
    private String stringValue = null;
    /** Date start value */
    private Date dateStartValue = null;
    /** Date End value */
    private Date dateEndValue = null;
    /** int value */
    private int intValue = -1;
    /** Long value */
    private long longValue = -1l;
    /** float value */
    private float floatValue = -1f;
    /** double value */
    private double doubleValue = -1d;
    /** Boolean value */
    private boolean booleanValue = false;
    
    /**
     * Build a string based selector
     * @param s value
     */
    public ConsolidateSelector(String s) {
        if (s == null) {
            throw new IllegalArgumentException("Cannot accept null as selector");
        }
        this.type = ValueType.STRING;
        this.stringValue = s;
    }
    
    /**
     * Build a range date based selector.
     * @param min date minimum date limit, included
     * @param max date maximum date limit, excluded
     * @param ref date reference date: the range must include this date
     * @param rangeType type of range
     */
    public ConsolidateSelector(Date min, Date max, Date ref, DateRangeType rangeType) {
        // Basically, I have to define a range that contains the ref. date.
        // First of all, I build the range, despite limits
        Date s = null;
        Date e = null;
        // Rounding to day anyway.
        ref = DateUtils.truncate(ref, Calendar.DAY_OF_MONTH);
        switch (rangeType) {
        case NONE:
            s = min;
            e = max;
            break;
        case DAILY:
            s = ref;
            e = DateUtils.add(ref, Calendar.DAY_OF_MONTH, 1);
            break;
        case WEEKLY:
            Calendar refCal = Calendar.getInstance();
            refCal.setTime(ref);
            int dayOfWeek = refCal.get(Calendar.DAY_OF_WEEK);
            s = DateUtils.add(ref, Calendar.DAY_OF_MONTH, -1 * (dayOfWeek - 1));
            e = DateUtils.add(ref, Calendar.DAY_OF_MONTH, (7 - dayOfWeek) + 1);
            break;
        case MONTHLY:
            s = DateUtils.truncate(ref, Calendar.MONTH);
            e = DateUtils.add(s, Calendar.MONTH, 1);
            break;
        default:
            throw new IllegalStateException("Impossible");
        }
        // Now applying limits
        if (s.before(min)) {
            s = min;
        }
        if (e.after(max)) {
            e = max;
        }
        // Then init
        this.setDateRange(s, e);
    }
    
    /**
     * Build a date based selector
     * @param ds date start and end value
     */
    public ConsolidateSelector(Date d) {
        this.setDateRange(d);
    }
    
    /**
     * Build a date based selector
     * @param ds date start and end value
     */
    private void setDateRange(Date d) {
        this.setDateRange(d, d);
    }
    /**
     * Build a date based selector
     * @param ds date start value
     * @param de date end value
     */
    private void setDateRange(Date ds, Date de) {
        if (ds == null || de == null) {
            throw new IllegalArgumentException("Cannot accept null as selector");
        }
        this.type = ValueType.DATE_RANGE;
        this.dateStartValue = ds;
        this.dateEndValue = de;
    }
    
    /**
     * Build a integer based selector
     * @param s value
     */
    public ConsolidateSelector(int s) {
        this.type = ValueType.INT;
        this.intValue = s;
    }
    
    /**
     * Build a long based selector
     * @param s value
     */
    public ConsolidateSelector(long s) {
        this.type = ValueType.LONG;
        this.longValue = s;
    }
    
    /**
     * Build a float based selector
     * @param s value
     */
    public ConsolidateSelector(float s) {
        this.type = ValueType.FLOAT;
        this.floatValue = s;
    }
    
    /**
     * Build a double based selector
     * @param s value
     */
    public ConsolidateSelector(double s) {
        this.type = ValueType.DOUBLE;
        this.doubleValue = s;
    }
    
    /**
     * Build a boolean based selector
     * @param s value
     */
    public ConsolidateSelector(boolean s) {
        this.type = ValueType.BOOLEAN;
        this.booleanValue = s;
    }
    
    
    /**
     * @return the value type of this selector
     */
    ValueType getType() {
        return this.type;
    }
    
    /**
     * @return the value as string
     * @throws IllegalStateException if the type is not STRING
     */
    String getStringValue() {
        if (this.type != ValueType.STRING) {
            throw new IllegalStateException("The selector type is not STRING");
        }
        return this.stringValue;
    }
    /**
     * @return the value as date (start value)
     * @throws IllegalStateException if the type is not DATE_RANGE
     */
    Date getDateStartValue() {
        if (this.type != ValueType.DATE_RANGE) {
            throw new IllegalStateException("The selector type is not DATE_RANGE");
        }
        return this.dateStartValue;
    }
    /**
     * @return the value as date (end value)
     * @throws IllegalStateException if the type is not DATE_RANGE
     */
    Date getDateEndValue() {
        if (this.type != ValueType.DATE_RANGE) {
            throw new IllegalStateException("The selector type is not DATE_RANGE");
        }
        return this.dateEndValue;
    }
    /**
     * @return the value as integer
     * @throws IllegalStateException if the type is not INT
     */
    int getIntValue() {
        if (this.type != ValueType.INT) {
            throw new IllegalStateException("The selector type is not INT");
        }
        return this.intValue;
    }
    /**
     * @return the value as long integer
     * @throws IllegalStateException if the type is not LONG
     */
    long getLongValue() {
        if (this.type != ValueType.LONG) {
            throw new IllegalStateException("The selector type is not LONG");
        }
        return this.longValue;
    }
    /**
     * @return the value as float decimal
     * @throws IllegalStateException if the type is not FLOAT
     */
    float getFloatValue() {
        if (this.type != ValueType.FLOAT) {
            throw new IllegalStateException("The selector type is not FLOAT");
        }
        return this.floatValue;
    }
    /**
     * @return the value as double decimal
     * @throws IllegalStateException if the type is not DOUBLE
     */
    double getDoubleValue() {
        if (this.type != ValueType.DOUBLE) {
            throw new IllegalStateException("The selector type is not DOUBLE");
        }
        return this.doubleValue;
    }
    /**
     * @return the value as boolean
     * @throws IllegalStateException if the type is not BOOLEAN
     */
    boolean getBooleanValue() {
        if (this.type != ValueType.BOOLEAN) {
            throw new IllegalStateException("The selector type is not BOOLEAN");
        }
        return this.booleanValue;
    }

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(ConsolidateSelector o) {
        if (o == null) {
            return 1;
        }
        if (o.type != this.type) {
            return this.type.compareTo(o.type);
        } else {
            switch (this.type) {
            case BOOLEAN:
                return this.booleanValue == o.booleanValue ? 0 : 1;
            case DOUBLE:
                return new Double(this.doubleValue).compareTo(new Double(o.doubleValue));
            case DATE_RANGE:
                return this.dateStartValue.compareTo(o.dateStartValue) + this.dateEndValue.compareTo(o.dateEndValue);
            case FLOAT:
                return new Float(this.floatValue).compareTo(new Float(o.floatValue));
            case INT:
                return this.intValue - o.intValue;
            case LONG:
                return new Long(this.longValue).compareTo(new Long(o.longValue));
            case STRING:
                return this.stringValue.compareTo(o.stringValue);
            default:
                throw new IllegalStateException("Impossible");
            }
        }
    }
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof ConsolidateSelector)) {
            return false;
        }
        return this.compareTo((ConsolidateSelector) obj) == 0;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        int res = 0 + this.type.hashCode();
        switch (this.type) {
        case BOOLEAN:
            return res + (this.booleanValue ? 1 : 2);
        case DOUBLE:
            return res + Double.toString(this.doubleValue).hashCode();
        case DATE_RANGE:
            return res + this.dateStartValue.hashCode() + this.dateEndValue.hashCode();
        case FLOAT:
            return res + Float.toString(this.floatValue).hashCode();
        case INT:
            return res + this.intValue;
        case LONG:
            return res + Long.toString(this.longValue).hashCode();
        case STRING:
            return res + this.stringValue.hashCode();
        default:
            throw new IllegalStateException("Impossible");
        }
    }

    /**
     * @return a string to be displayed to a end user
     */
    public String toGUIViewString() {
        switch (this.type) {
        case BOOLEAN:
            return String.valueOf(this.booleanValue);
        case DOUBLE:
            return String.valueOf(this.doubleValue);
        case DATE_RANGE:
            // The end date is excluded. To have the date included, simply remove one day
            Date end = DateUtils.add(this.dateEndValue, Calendar.DAY_OF_MONTH, -1);
            StringBuffer dtString = new StringBuffer();
            dtString.append(DateFormatUtils.format(this.dateStartValue, DATE_FORMAT_PATTERN));
            // If start date and end date are equals, well, just display one date ;)
            if (!this.dateStartValue.equals(end)) {
                dtString.append(" to ");
                dtString.append(DateFormatUtils.format(end, DATE_FORMAT_PATTERN));
            }
            return dtString.toString();
        case FLOAT:
            return String.valueOf(this.floatValue);
        case INT:
            return String.valueOf(this.intValue);
        case LONG:
            return String.valueOf(this.longValue);
        case STRING:
            return this.stringValue;
        default:
            throw new IllegalStateException("Impossible");
        }
    }

}
