package com.jujunie.service.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.ArrayUtils;

import com.jujunie.service.log.Log;

/**
 * This class convert a stream of bytes to a stream of text.
 * @author julien
 *
 */
public class BytesToText {

    /** Logger */
    Log log = Log.getInstance(BytesToText.class);
    
    /** Base buffer size */
    private static int BASE_BUFFER_SIZE = 1024;
    
    private int bufferSize = 0;
    /** Output as writer for Byte 2 Text*/
    private Writer outWriter = null;
    /** Output stream for text 2 bytes */
    private OutputStream outStream = null;
    /** Charset decoder */
    private CharsetDecoder charDecode = null;
    /** Charset encoder */
    private CharsetEncoder charEncode = null;
    /** Byte buffer */
    private ByteBuffer bb = null;
    /** Char buffer */
    private CharBuffer cb = null;
    /** Flag to indicate the first call to decode */
    private boolean decodeCalled = false;
    /** Flag to indicate the first call to encode */
    private boolean encodeCalled = false;
    
    /**
     * Constructor
     * @param out output writer
     */
    public BytesToText(Writer out, int bufferSize) {
        if (out == null) {
            throw new IllegalArgumentException("Cannot accept null as output writer !");
        }
        this.outWriter = out;
        this.charDecode = Charset.forName("US-ASCII").newDecoder();
        this.bb = ByteBuffer.allocate(1600);
        this.cb = CharBuffer.allocate(1600);
        this.init(bufferSize);
    }
    
    public BytesToText(OutputStream out, int bufferSize) {
        if (out == null) {
            throw new IllegalArgumentException("Cannot accept null as input stream !");
        }
        this.outStream = out;
        this.charEncode = Charset.forName("US-ASCII").newEncoder();
        this.bb = ByteBuffer.allocate(1200);
        this.cb = CharBuffer.allocate(1200);
        this.init(bufferSize);
    }
    
    private void init(int bufferSize) {
        this.bufferSize = (bufferSize > 0 ? bufferSize : BASE_BUFFER_SIZE);
        this.decodeCalled = false;
    }
    
    public void decode(Reader in) throws IOException {
        log.enter("decode(Reader)");
        if (in == null) {
            throw new IllegalArgumentException("Cannot accept null as input stream reader!");
        }
        char[] buffer = new char[1200];
        int read = 0;
        do {
            read = in.read(buffer);
            if (read <= 0) {
                log.debug("Nothing remaining to read => break");
                break;
            }
            this.decode(buffer, 0, read);
        } while (true);
        log.exit("decode(Reader)");
    }
    
    public void decode(char[] ch, int start, int length) throws IOException {
        log.enter("decode(char[], int, int)");
        if (this.decodeCalled) {
            log.debug("Already called => decoding");
            this.decode(false);
        } else {
            log.debug("First call => buffering");
            // Changing flag
            this.decodeCalled = true;
        }
        // Buffering anyway...
        this.cb.put(ch, start, length);
        log.exit("decode(char[], int, int)");
    }
    
    private void decode(boolean endOfInput) throws IOException {
        this.cb.flip();
        // char to bytes
        this.charEncode.encode(this.cb, this.bb, endOfInput);
        // Compacting cb...
        this.cb.compact();
        this.writeToOutStream();
    }
    
    private void writeToOutStream() throws IOException {
        this.bb.flip();
        byte[] toWrite = ArrayUtils.subarray(this.bb.array(), this.bb.position(), this.bb.limit());
        // Decoding bb, and write...
        this.outStream.write(Base64.decodeBase64(toWrite));
        this.bb.clear();
    }
    
    public void complete() throws IOException {
        log.enter("complete");
        if (this.outWriter != null) {
            // One last call...
            this.encode(true);
            // One last call...
            this.charDecode.flush(this.cb);
            this.writeToWriter();
        }
        if (this.outStream != null) {
            // One last call...
            this.decode(true);
            // flush encoder
            this.charEncode.flush(this.bb);
            this.writeToOutStream();
            // flushing
            this.outStream.flush();
        }
        log.exit("complete");
    }
    
    /**
     * Encode the given input stream in the writer given in the constructor
     * @param in stream of bytes
     * @throws IOException on IO errors
     */
    public void encode(InputStream in) throws IOException {
        log.enter("encode(InputStream)");
        if (in == null) {
            throw new IllegalArgumentException("Cannot accept null as input stream !");
        }
        byte[] readBuffer = new byte[1200];
        int read = 0;
        do {
            // Reading some bytes
            read = in.read(readBuffer);
            log.debug("ENCODE: Read bytes: ",  read);
            if (read <= 0) {
                log.debug("Nothing else to read => break");
                break;
            }
            this.encode(readBuffer, 0, read);
        } while (read > 0);
        log.exit("encode(InputStream)");
    }
    
    public void encode(byte[] by, int start, int length) throws IOException {
        log.enter("encode(byte[], int, int)");
        if (this.encodeCalled) {
            log.debug("Encode already called => encoding");
            this.encode(false);
        } else {
            log.debug("First call => buffering");
            // Changing flag
            this.encodeCalled = true;
        }
        // Buffering anyway...
        by = Base64.encodeBase64(ArrayUtils.subarray(by, start, length));
        log.debug("Encoded size: " + by.length);
        log.debug("Writing in byte buffer");
        this.bb.put(by);
        log.exit("encode(byte[], int, int)");
    }
    
    private void encode(boolean endOfInput) throws IOException {
        log.enter("encode(boolean)");
        log.debug("End of output? " + endOfInput);
        log.debug("Preparing byte buffer for reading");
        this.bb.flip();
        // Byte to char ...
        this.charDecode.decode(this.bb, this.cb, endOfInput);
        log.debug("Preparing byte buffer for writing");
        // Compacting bb, to place remaining bytes at the beginning of the buffer
        this.bb.compact();
        this.writeToWriter();
        log.exit("encode(boolean)");
    }
    
    private void writeToWriter() throws IOException {
        log.enter("writeToWriter");
        log.debug("Preparing byte buffer for reading");
        this.cb.flip();
        // Writing what have been writen to the char buffer
        log.debug("Char buffer position: " + this.cb.position());
        log.debug("Char buffer limit: " + this.cb.limit());
        this.outWriter.write(this.cb.array(), this.cb.position(), this.cb.limit());
        log.debug("Clearing char buffer");
        this.cb.clear();
        log.exit("writeToWriter");
    }
    
}
