/*
 * Created on 19 nov. 2003
 *
 */
package com.jujunie.service.web;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.jujunie.service.ApplicationContext;
import com.jujunie.service.log.Log;
import com.jujunie.service.web.html.HtmlElementsProvider;
import com.jujunie.service.web.html.HtmlElementsProviderXML;
import com.jujunie.service.xml.InvalidXMLElementException;

/**
 * This is an implementation of the WebApplication interface
 * loaded using an XML DOM structure.
 * Required attributes :
 * <ul>
 *   <li>code : Code to identify the application</li>
 *   <li>default-module : code of the default module of this application</li>
 * </ul>
 * Optionnal attributes :
 * <ul>
 *   <li>name : Full name of the application</li>
 * </ul>
 * Nested elements :
 * <ul>
 *   <li> module (0 to n). See WebModuleXML</li>
 *   <li> template (0 to n). See WebTemplateXML</li>
 * </ul>
 * @see WebModuleXML 
 * @author <a href="mailto:julien@jujunie.com">julien</a>
 */
public class WebApplicationXML extends WebElementXML implements WebApplication, Serializable {

    /** serialVersionUID */
    private static final long serialVersionUID = 808323050068464490L;

    /** Application element name */
    public static final String ELEMENT_NAME = "application";
    /** WebTransactionHandler element name */
    public static final String WEB_TRANSACTION_HANDLER_ELEMENT_NAME = "transaction-handler";
    
    /** Application name */
    private String name = "";
    
    /** Application code */
    private String code = "";
    
    /** Web config */
    private WebConfig parent;
    
    /** Flag to indicate that this application is active or not */
    private boolean active = false;
    
    /** Associated modules */
    private Map<String, WebModule> modules;
    
    /** Associated templates */
    private Map<String, WebTemplate> templates;
    
    /** Params */
    private Map<String, WebProperty> params;
    
    /** Default module */
    private WebModule defaultModule;
    
    /** Default template */
    private WebTemplate defaultTemplate;
    
    /** Init command */
    private InitCommand initCommand = null;
    
    /** Html provider */
    private HtmlElementsProviderXML htmlElements = null;
    
    /** Application context */
    private ApplicationContext context = new ApplicationContext() {
        /** serialVersionUID */
		private static final long serialVersionUID = 1938270753911893673L;
		/** Map */
        private Map<String, Object> atts = new HashMap<String, Object>();
        /**
         * @see com.jujunie.service.ApplicationContext#setAttribute(java.lang.String, java.lang.Object)
         */
        public void setAttribute(String key, Object att) {
            this.atts.put(key, att);
        }

        /**
         * @see com.jujunie.service.ApplicationContext#getAttribute(java.lang.String)
         */
        public Object getAttribute(String key) {
            return this.atts.get(key);
        }
    };
    /** WebTransactionHandler class name*/
    private WebTransactionHandler webTransactionHandler =  WebTransactionHandler.DEFAULT;
    
    /**
     * Loads associated modules
     * @param e the application element in the XML file
     * @throws InvalidXMLElementException
     */
    private void loadModules(Element e)
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        Log.getInstance(this).enter("loadModules");
        NodeList mods = super.getChildsNamed(e, WebModuleXML.ELEMENT_NAME);
        WebModule current;
        this.modules = new HashMap<String, WebModule>(mods.getLength());
        for (int i = 0; i < mods.getLength(); i++) {
            current = new WebModuleXML(mods.item(i), this);
            Log.getInstance(this).debug("Module " + current.getCode() + " loaded");
            this.modules.put(current.getCode(), current);
        }
        Log.getInstance(this).exit("loadModules");
    }
    
    /**
     * Loads associated templates
     * @param e the application element in the XML file
     * @throws InvalidXMLElementException
     */
    private void loadTemplates(Element e) throws InvalidXMLElementException {
        Log.getInstance(this).enter("loadTemplates");
        NodeList mods = super.getChildsNamed(e, WebTemplateXML.ELEMENT_NAME);
        WebTemplate current;
        this.templates = new HashMap<String, WebTemplate>(mods.getLength());
        for (int i = 0; i < mods.getLength(); i++) {
            current = new WebTemplateXML(mods.item(i), this);
            Log.getInstance(this).debug("Template " + current.getCode() + " loaded");
            this.templates.put(current.getCode(), current);
        }
        Log.getInstance(this).exit("loadTemplates");
    }
    
    /**
     * Loads parameter
     * @param e the application element in the XML file
     * @throws InvalidXMLElementException
     */
    private void loadParameters(Element e) throws InvalidXMLElementException {
        Log.getInstance(this).enter("loadParameters");
        NodeList mods = super.getChildsNamed(e, WebPropertyXML.ELEMENT_NAME);
        WebProperty current;
        this.params = new HashMap<String, WebProperty>(mods.getLength());
        for (int i = 0; i < mods.getLength(); i++) {
            current = new WebPropertyXML(mods.item(i));
            Log.getInstance(this).debug("Parameter loaded :  "
                    + current.getCode() + "=" + current.getValue());
            this.params.put(current.getCode(), current);
        }
        Log.getInstance(this).exit("loadParameters");
    }
    
    /**
     * Setting the default module of this application.
     * @param appElement the XML application element
     * @throws WebItemNotAvailableException if the default module is not found
     * @throws InvalidXMLElementException if the application element is not correct
     */
    private void setDefaultModule(Element appElement) throws WebItemNotAvailableException, InvalidXMLElementException {
        String defaultModuleName;
        defaultModuleName = super.getAttribute(appElement, ATT_DEFAULT_MODULE, true);
        this.defaultModule = (WebModule) this.getModule(defaultModuleName);
    }
    
    /**
     * Setting the default template of this application.
     * @param appElement the XML application element
     * @throws WebItemNotAvailableException if the default module is not found
     * @throws InvalidXMLElementException if the application element is not correct
     */
    private void setDefaultTemplate(Element appElement)
    throws WebItemNotAvailableException, InvalidXMLElementException {
        String defaultTemplateName;
        defaultTemplateName = super.getAttribute(appElement, ATT_DEFAULT_TEMPLATE, true);
        // Checking if the given default module exists
        if (!this.templates.containsKey(defaultTemplateName)) {
            throw new WebItemNotAvailableException("The given default template : "
            + defaultTemplateName + " do not exists");
        }
        this.defaultTemplate = (WebTemplate) this.templates.get(defaultTemplateName);
    }
    
    
    /**
     * Loads the init command 
     * @param appElement web application node
     * @throws InvalidXMLElementException if one XML element is invalid
     * @throws WebItemNotAvailableException if an item is unaivailable
     * @throws WebConfigException if a error occurs during dynamic class loading
     */
    private void setInitCommand(Element appElement) 
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        NodeList commandsList = appElement.getElementsByTagName(InitCommandXML.ELEMENT_NAME);
        if (commandsList.getLength() == 0) {
            this.initCommand = DummyCommand.getInstance();
        } else {
            Element currElt = (Element) commandsList.item(0);
            String className = super.getAttribute(currElt, ATT_CLASS, true);
            InitCommandXML current = null;
            try {
                current = (InitCommandXML) Class.forName(className).newInstance();
            } catch (IllegalAccessException e) {
                String message = "Insufficient rights to access " + className + ". IllegalAccessException occurs";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (ClassCastException e) {
                String message = "The class " + className + " do not implement com.jujunie.service.web.InitCommandXML";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (ClassNotFoundException e) {
                String message = "The class " + className + " cannot be found. Please check your classpath";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (InstantiationException e) {
                String message = "The class " + className + " do not have a public empty constructor";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            }
            current.init(currElt, this);
            this.initCommand = current;
        }
    }
    
    private void setWebTransactionHandler(Element appElement) 
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        NodeList commandsList = appElement.getElementsByTagName(WEB_TRANSACTION_HANDLER_ELEMENT_NAME);
        if (commandsList.getLength() != 0) {
            Element currElt = (Element) commandsList.item(0);
            String className = super.getAttribute(currElt, ATT_CLASS, true);
            try {
                this.webTransactionHandler = ( WebTransactionHandler) Class.forName(className).newInstance();
            } catch (IllegalAccessException e) {
                String message = "Insufficient rights to access " + className + ". IllegalAccessException occurs";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (ClassCastException e) {
                String message = 
                        "The class " + className + " do not implement com.jujunie.service.web.WebTransactionHandler";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (ClassNotFoundException e) {
                String message = "The class " + className + " cannot be found. Please check your classpath";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (InstantiationException e) {
                String message = "The class " + className + " do not have a public empty constructor";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            }
        }
    }
    
    /**
     * Load the HTML elements configuration
     * @param appElement application XML noe
     * @throws InvalidXMLElementException if one XML element is invalid
     * @throws WebConfigException if a error occurs during dynamic class loading
     */
    private void setHtmlElements(Element appElement)
    throws InvalidXMLElementException, WebConfigException {
        NodeList html = appElement.getElementsByTagName(HtmlElementsProviderXML.HTML_NODE_NAME);
        if (html.getLength()> 0) {
            this.htmlElements = new HtmlElementsProviderXML((Element) html.item(0));
        }
    }
    
    /**
     * Constructor
     * @param app the XM node representing the application
     * @throws InvalidXMLElementException
     * @throws WebItemNotAvailableException if the default module is not found
     */
    protected WebApplicationXML(Node app, WebConfig parent)
    throws WebItemNotAvailableException, InvalidXMLElementException, WebConfigException {
        Log.getInstance(this).enter("WebApplicationXML");
        Element appElement;
        this.parent = parent;
        if (app.getNodeType() != Node.ELEMENT_NODE) {
            throw new InvalidXMLElementException("The given XML node is not an element");
        }
        appElement = (Element) app;
        if (!appElement.getTagName().equals(ELEMENT_NAME)) {
            throw new InvalidXMLElementException("The given Element is not a " + ELEMENT_NAME
                                               + " element");
        }
        this.code = super.getAttribute(appElement, ATT_CODE, true);
        Log.getInstance(this).debug("Code : " + this.code);
        this.name = super.getAttribute(appElement, ATT_NAME, true);
        Log.getInstance(this).debug("Name : " + this.name);
        this.active = super.getAttribute(appElement, ATT_ACTIVE, false).equals("true");
        Log.getInstance(this).debug("Active : " + this.active);
        // Now loading templates
        this.loadTemplates(appElement);
        // Setting default template
        this.setDefaultTemplate(appElement);
        // Now loading modules
        this.loadModules(appElement);
        // Now loading parameters
        this.loadParameters(appElement);
        // Now loading init command
        this.setInitCommand(appElement);
        // Setting default module
        this.setDefaultModule(appElement);
        // Loading html elements
        this.setHtmlElements(appElement);
        // Loading the transaction handler
        this.setWebTransactionHandler(appElement);
        Log.getInstance(this).exit("WebApplicationXML");
    }

    /**
     * Execute operations that can be executed only when the
     * complete configuration have been loaded
     */
    public void onWebConfigLoaded() throws WebItemNotAvailableException {
        //Calling onWebConfigLoaded of childs
        for (Iterator i = this.modules.values().iterator(); i.hasNext();) {
            ((WebModuleXML) i.next()).onWebConfigLoaded();
        }
    }
    
    /**
     * @see com.jujunie.service.web.WebApplication#getName()
     */
    public String getName() {
        return this.name;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getCode()
     */
    public String getCode() {
        return this.code;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getDefaultModule()
     */
    public WebModule getDefaultModule() {
        return this.defaultModule;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getModule(java.lang.String)
     */
    public WebModule getModule(String modulePath) throws WebItemNotAvailableException {
        WebModule res;
        String moduleName = StringUtils.defaultString(StringUtils.substringBefore(modulePath, "/"));
        String path = StringUtils.defaultString(StringUtils.substringAfter(modulePath, "/"));
        res = (WebModule) this.modules.get(moduleName);
        if (res == null) {
            throw new WebItemNotAvailableException("The module named " + moduleName + " not found");
        }
        if (path.length() == 0) {
            return res;
        } else {
            return res.getModule(path);
        }
    }

    /**
     * @see com.jujunie.service.web.WebApplication#isActive()
     */
    public boolean isActive() {
        return this.active;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getDefaultTemplate()
     */
    public WebTemplate getDefaultTemplate() {
        return this.defaultTemplate;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getTemplate(java.lang.String)
     */
    public WebTemplate getTemplate(String templateName) throws WebItemNotAvailableException {
        WebTemplate res = (WebTemplate) this.templates.get(templateName);
        if (res == null) {
            String message = "The template named \"" + templateName + "\" cannot be found";
            Log.getInstance(this).error(message);
            throw new WebItemNotAvailableException(message);
        }
        return res;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getWebProperty(java.lang.String)
     */
    public WebProperty getWebProperty(String code) {
        return (WebProperty) this.params.get(code);
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getInitCommand()
     */
    public InitCommand getInitCommand() {
        return this.initCommand;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getApplicationContext()
     */
    public ApplicationContext getApplicationContext() {
        return this.context;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getHtmlElements()
     */
    public HtmlElementsProvider getHtmlElements() {
        return this.htmlElements;
    }

    /**
     * @see com.jujunie.service.web.WebApplication#getWebConfig()
     */
    public WebConfig getWebConfig() {
        return this.parent;
    }

    public WebTransactionHandler getWebTransactionHandler() {
        return this.webTransactionHandler;
    }

}
