/*
 * Created on 11 nov. 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.jujunie.service.web;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.jujunie.service.log.Log;
import com.jujunie.service.xml.*;

/**
 * This is a WebConfig implementation which uses an 
 * XML document to be loaded.
 * @author <a href="mailto:julien@jujunie.com">julien</a>
 */
public class WebConfigXML extends WebElementXML implements WebConfig {
    
    /** serialVersionUID */
    private static final long serialVersionUID = -8066070799236858405L;

    /** Map of Applications configured */
    private Map<String, WebApplication> applications;
    
    /** Display Handler */
    private Map<String, DisplayHandlerXML> displays = 
            new HashMap<String, DisplayHandlerXML>();
    
    /** Default Display Handler */
    private DisplayHandlerXML defaultDisplay;
    
    /** Default application */
    private WebApplication defaultApp;
    

    
    /**
     * Init the web configuration, and builds the navigation three
     * @param xmlConfig
     * @throws WebConfigException if problem occurs during initalisation
     */
    public WebConfig init(InputStream xmlConfig) throws WebConfigException {
        Log.getInstance(this).enter("init");
        Document doc = null;
        Element root = null;
        NodeList list = null;
        try {
            doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(xmlConfig);
        } catch (ParserConfigurationException e) {
            String message = "ParserConfigurationException caught when trying to parse the web xml configuration file."
                           + " Parsing configuration may not be correct";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (SAXException e) {
            String message = "SAXException caught when trying to parse the web xml configuration file."
                           + " The given XML may not be correct";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (IOException e) {
            String message = "IOException caught when trying to parse the web xml configuration file."
                           + " The resource may not be available / readable";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        }
        // Getting root element
        if (doc != null) {
            try {
                root = doc.getDocumentElement();
                Log.getInstance(this).debug("Configuration document loaded. Loading display handlers");
                // Getting the display handler
                list = root.getElementsByTagName(DisplayHandlerXML.ELEMENT_NAME);
                this.setDisplayHandler(list);
                // Getting the default display handler
                this.setDefaultDisplay(root);
                Log.getInstance(this).debug("Display handlers loaded. Loading applications");
                // --- Creating a WebApplication object for each application element
                list = root.getElementsByTagName(WebApplicationXML.ELEMENT_NAME);
                this.setApplications(list);
                // Getting default application
                this.setDefaultApp(root);
            } catch (InvalidXMLElementException e) {
                String message = "Structure error while loading applications : "
                               + e.getMessage();
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (WebItemNotAvailableException e) {
                String message = "Structure error while loading applications : "
                               + e.getMessage();
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            }
        } else {
            Log.getInstance(this).warn("The document element is empty. No configuration loaded");
        }
        Log.getInstance(this).exit("init");
        return this;
    }
    
    /**
     * Set the default application
     * @param e root element of web configuration
     * @throws InvalidXMLElementException if cannot found the default application attribute
     * @throws WebConfigException  if cannot found the default application in loaded application
     */
    private void setDefaultApp(Element e) throws InvalidXMLElementException, WebConfigException {
        String appName = this.getAttribute(e, ATT_DEFAULT_APP, true);
        if (this.applications.containsKey(appName)) {
            this.defaultApp = this.applications.get(appName);
        } else {
            throw new WebConfigException("Default application " + appName
                                       + " cannot be found", null);
        }
    }
    
    /**
     * Set the default display handler
     * @param e root element of web configuration
     * @throws InvalidXMLElementException if cannot found the default
     * display attribute
     * @throws WebConfigException  if cannot found the default display 
     * in loaded displays
     */
    private void setDefaultDisplay(Element e) throws InvalidXMLElementException, WebConfigException {
        Log.getInstance(this).enter("setDefaultDisplay(Element)");
        String diName = this.getAttribute(e, ATT_DEFAULT_DI, true);
        if (this.displays.containsKey(diName)) {
            this.defaultDisplay = this.displays.get(diName);
        } else {
            throw new WebConfigException("Default display handler " + diName
                                       + " cannot be found", null);
        }
        Log.getInstance(this).debug("Default display handler attached: ", this.defaultDisplay.getCode());
        Log.getInstance(this).exit("setDefaultDisplay(Element)");
    }
    
    private void setDisplayHandler(NodeList displayHandlerItem)
    throws InvalidXMLElementException,
            WebItemNotAvailableException,
            WebConfigException {
        Log.getInstance(this).enter("setDisplayHandler(NodeList)");
        if (displayHandlerItem.getLength() == 0) {
            String message = "There is no display handlers defined."
                           + " Please correct your web configuration file";
            Log.getInstance(this).error(message);
            throw new WebItemNotAvailableException(message);
        }
        // Getting the display handler
        Element elt = null;
        String className = null;
        DisplayHandlerXML dh = null;
        try {
            for (int i = 0; i < displayHandlerItem.getLength(); i++) {
                elt = (Element) displayHandlerItem.item(i);
                className = super.getAttribute(elt, ATT_CLASS, true);
                dh = (DisplayHandlerXML) Class.forName(className).newInstance();
                dh.init(elt);
                this.displays.put(dh.getCode(), dh);
                Log.getInstance(this).debug("Display Handler loaded: ", dh.getCode());
            }
        } catch (IllegalAccessException e) {
            String message = 
                    "Insufficient rights to access " + className + 
                    ". IllegalAccessException occurs";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (ClassCastException e) {
            String message = "The class " + className + 
                    " do not implement" +
                    " com.jujunie.service.web.DisplayHandlerXML";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (ClassNotFoundException e) {
            String message = "The class " + className + 
                    " cannot be found. Please check your classpath";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (InstantiationException e) {
            String message = "The class " + className + 
                    " do not have a public empty constructor";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        }
        Log.getInstance(this).exit("setDisplayHandler(NodeList)");
    }
    
    /**
     * Set the web applications
     * @param apps
     * @throws WebConfigException
     */
    private void setApplications(NodeList apps)
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        WebApplicationXML current = null;
        this.applications = new HashMap<String, WebApplication>(apps.getLength());
        for(int i = 0; i < apps.getLength(); i++) {
            current = new WebApplicationXML(apps.item(i), this);
            current.onWebConfigLoaded();
            Log.getInstance(this).info("Application " + current.getName() + " loaded");
            this.applications.put(current.getCode(), current);
        }
    }

    /**
     * @see com.jujunie.service.web.WebConfig#getDefaultApplication()
     */
    public WebApplication getDefaultApplication() {
        return this.defaultApp;
    }

    /**
     * @see com.jujunie.service.web.WebConfig#getDisplayHandler(java.lang.String) 
     */
    public DisplayHandler getDisplayHandler(String code) {
        return this.displays.get(code);
    }

    /**
     * @see com.jujunie.service.web.WebConfig#getDefaultDisplayHandler() 
     */
    public DisplayHandler getDefaultDisplayHandler() {
        return this.defaultDisplay;
    }

}
