/*
 * Created on Nov 29, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.jujunie.service.web;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.jujunie.service.log.Log;
import com.jujunie.service.xml.*;

/**
 * This is an implementation of the WebModule interface
 * loaded using an XML DOM structure.
 * Nested into a application or module element.
 * Required attributes :
 * <ul>
 *   <li>code : code of the module</li>
 * </ul>
 * Optionnal attributes :
 * <ul>
 *   <li>default-page : code of the default page for this module</li>
 * </ul>
 * Nested elements :
 * <ul>
 *   <li> module (0 to n). See WebModuleXML</li>
 *   <li> page (0 to n). See WebPageXML</li>
 * </ul>
 * @see WebPageXML
 * @author julien
 */
public class WebModuleXML extends WebElementXML implements WebModule, Serializable {

    /** serialVersionUID */
    private static final long serialVersionUID = 7398653893851070865L;

    /** Module element name */
    public static final String ELEMENT_NAME = "module";
    
    /** Module code */
    private String code = "";
    
    /** Application linked to this module */
    private WebApplication application;
    
    /** Associated modules */
    private Map<String, WebModule> modules = Collections.emptyMap();
    
    /** Associated pages */
    private Map<String, WebPage> pages = Collections.emptyMap();
    
    /** Default page */
    private WebPage defaultPage;
    
    /**
     * Constructor
     * @param mod the XM node representing the module
     * @param app parent application
     * @throws InvalidXMLElementException
     */
    protected WebModuleXML(Node mod, WebApplication app)
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        Log.getInstance(this).enter("WebModuleXML");
        Element modElement;
        this.application = app;
        if (mod.getNodeType() != Node.ELEMENT_NODE) {
            throw new InvalidXMLElementException("The given XML node is not an element");
        }
        modElement = (Element) mod;
        if (!modElement.getTagName().equals(ELEMENT_NAME)) {
            throw new InvalidXMLElementException("The given Element is not a " + ELEMENT_NAME
                                               + " element");
        }
        this.code = super.getAttribute(modElement, ATT_CODE, true);
        Log.getInstance(this).debug("Code : " + this.code);
        // Now loading pages...
        this.loadPages(modElement);
        // Getting default page
        this.defaultPage = this.getPage(super.getAttribute(modElement, ATT_DEFAULT_PAGE, true));
        if (this.defaultPage == null) {
            throw new InvalidXMLElementException("Cannot found default page for module : " + this.code);
        } else {
            Log.getInstance(this).debug("Default page found : " + this.code);
        }
        // Now loading sub modules
        this.loadSubModules(modElement);
        Log.getInstance(this).exit("WebModuleXML");
    }
    
    /**
     * Loads associated pages
     * @param e the module element in the XML file
     * @throws InvalidXMLElementException
     */
    private void loadPages(Element e)
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        Log.getInstance(this).enter("loadPages");
        NodeList pages = e.getElementsByTagName(WebPageXML.ELEMENT_NAME);
        WebPage current;
        this.pages = new HashMap<String, WebPage>(pages.getLength());
        for (int i = 0; i < pages.getLength(); i++) {
            current = new WebPageXML(pages.item(i), this.application, this);
            Log.getInstance(this).debug("Page " + current.getCode() + " loaded");
            this.pages.put(current.getCode(), current);
        }
        Log.getInstance(this).exit("loadPages");
    }
    
    /**
     * Loads associated sub-modules
     * @param e the module element in the XML file
     * @throws InvalidXMLElementException
     */
    private void loadSubModules(Element e)
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        Log.getInstance(this).enter("loadSubModules");
        NodeList mods = e.getElementsByTagName(WebModuleXML.ELEMENT_NAME);
        WebModule current;
        if (mods.getLength() > 0) {
            this.modules = new HashMap<String, WebModule>(mods.getLength());
            for (int i = 0; i < mods.getLength(); i++) {
                current = new WebModuleXML(mods.item(i), this.application);
                Log.getInstance(this).debug("Sub module " + current.getCode() + " loaded");
                this.modules.put(current.getCode(), current);
            }
        }
        Log.getInstance(this).exit("loadSubModules");
    }
    
    /* (non-Javadoc)
     * @see com.jujunie.service.web.WebModule#getApplication()
     */
    public WebApplication getApplication() {
        return this.application;
    }

    /* (non-Javadoc)
     * @see com.jujunie.service.web.WebModule#getCode()
     */
    public String getCode() {
        return this.code;
    }

    /* (non-Javadoc)
     * @see com.jujunie.service.web.WebModule#getModule(java.lang.String)
     */
    public WebModule getModule(String modulePath) throws WebItemNotAvailableException {
        WebModule res;
        String moduleName = StringUtils.defaultString(StringUtils.substringBefore(modulePath, "/"));
        String path = StringUtils.defaultString(StringUtils.substringAfter(modulePath, "/"));
        res = (WebModule) this.modules.get(moduleName);
        if (res == null) {
            throw new WebItemNotAvailableException("The module named " + moduleName + " not found");
        }
        if (path.length() == 0) {
            return res;
        } else {
            return res.getModule(path);
        }
    }

    /**
     * Execute operations that can be executed only when the
     * complete configuration have been loaded
     */
    public void onWebConfigLoaded() throws WebItemNotAvailableException {
        //Calling onWebConfigLoaded of childs
        for (Iterator i = this.modules.values().iterator(); i.hasNext();) {
            ((WebModuleXML) i.next()).onWebConfigLoaded();
        }
        for (Iterator i = this.pages.values().iterator(); i.hasNext();) {
            ((WebPageXML) i.next()).onWebConfigLoaded();
        }
    }
    /**
     * @see com.jujunie.service.web.WebModule#getPage(java.lang.String)
     */
    public WebPage getPage(String code) throws WebItemNotAvailableException {
        return (WebPage) this.pages.get(code);
    }

    /**
     * @see com.jujunie.service.web.WebModule#getDefaultPage()
     */
    public WebPage getDefaultPage() {
        return this.defaultPage;
    }

}
