/*
 * Created on Nov 29, 2003
 *
 */
package com.jujunie.service.web;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.jujunie.service.log.Log;
import com.jujunie.service.xml.InvalidXMLElementException;

/**
 * This is an implementation of the WebPage interface
 * loaded using an XML DOM structure.
 * Nested into a module.
 * Required attributes :
 * <ul>
 *   <li>code : code of the page</li>
 * </ul>
 * Optional attributes :
 * <ul>
 *   <li>template : code of the template to use. If not set, the default
 *   template of the application will be used</li>
 * </ul>
 * Nested elements :
 * <ul>
 *   <li> area (0 to n). Defines the template overrrided areas</li>
 * </ul>
 * @see WebModuleXML
 * @see WebAreaXML
 * @author julien
 */
public class WebPageXML extends WebElementXML implements WebPage, Serializable {

    /** serialVersionUID */
    private static final long serialVersionUID = -7307064888729075964L;

    /** Page element name */
    public static final String ELEMENT_NAME = "page";
    
    /** Output Page element name */
    public static final String OUT_ELEMENT_NAME = "output";
    
    /** Encoding of the page */
    private String encoding = DEFAULT_ENCODING;
    
    /** Content type of the page */
    private String contentType = DEFAULT_CONTENT_TYPE; 
    
    /** Code of the page */
    private String code;
    
    /** Used template */
    private WebTemplate template;
    
    /** Area list */
    private Map<String, WebArea> areas;
    
    /** Command list */
    private Map<String, CommandXML> commands;
    
    /** Output pages list */
    private Map<String, WebPage> outputPages;
    
    /** Parent module */
    private WebModule parent;
    
    /** WebApplication */
    private WebApplication webApp = null;
    
    /** Temporary list to retreive output page once all the page are loaded */
    private List<String[]> tmpOutputPages;
    
    /** Display initialiser */
    private String displayInitialiser;
    
    /** Display Handler */
    private DisplayHandler displayHandler;
    
    
    /**
     * Loads the ouput pages configuration, waiting for complete loading
     * of pages
     * @param page the page element which contains the output elements
     * @throws InvalidXMLElementException if the XML is invalid
     */
    private void loadTemporaryOuputPages(Element page) throws InvalidXMLElementException {
        NodeList outputs = page.getElementsByTagName(OUT_ELEMENT_NAME);
        String name = "";
        String moduleCode = "";
        String pageCode = "";
        this.tmpOutputPages = new ArrayList<String[]>(outputs.getLength());
        Element currElt;
        for (int i = 0; i < outputs.getLength(); i++) {
            currElt = (Element) outputs.item(i);
            name = super.getAttribute(currElt, ATT_CODE, true);
            moduleCode = super.getAttribute(currElt, ATT_MODULE, false);
            pageCode = super.getAttribute(currElt, ATT_PAGE, false);
            this.tmpOutputPages.add(new String[] {name, moduleCode, pageCode});
        }
    }
    
    /**
     * Loads commands associated to this page
     * @param page the page element which contains the command elements
     * @throws InvalidXMLElementException if one XML element is invalid
     * @throws WebItemNotAvailableException if an item is unaivailable
     * @throws WebConfigException if a error occurs during dynamic class loading
     */
    private void loadCommands(Element page)
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        Log.getInstance(this).enter("loadCommands");
        NodeList commandsList = super.getChildsNamed(page, CommandXML.ELEMENT_NAME);
        CommandXML current;
        Element currElt;
        String className;
        this.commands = new HashMap<String, CommandXML>(commandsList.getLength());
        for (int i = 0; i < commandsList.getLength(); i++) {
            currElt = (Element) commandsList.item(i);
            className = super.getAttribute(currElt, ATT_CLASS, true);
            try {
                current = (CommandXML) Class.forName(className).newInstance();
            } catch (IllegalAccessException e) {
                String message = "Insufficient rights to access " + className + ". IllegalAccessException occurs";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (ClassCastException e) {
                String message = "The class " + className + " do not implement com.jujunie.service.web.CommandXML";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (ClassNotFoundException e) {
                String message = "The class " + className + " cannot be found. Please check your classpath";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            } catch (InstantiationException e) {
                String message = "The class " + className + " do not have a public empty constructor";
                Log.getInstance(this).error(message, e);
                throw new WebConfigException(message, e);
            }
            current.init(currElt, this);
            Log.getInstance(this).debug("Command " + current.getCode() + " loaded");
            this.commands.put(current.getCode(), current);
        }
        Log.getInstance(this).exit("loadCommands");
    }
    
    /**
     * Loads areas of the template and override with areas given
     * in page element.
     * @param page
     */
    private void loadAreas(Element page) throws InvalidXMLElementException, WebItemNotAvailableException {
        NodeList areasList = page.getElementsByTagName(WebAreaXML.ELEMENT_NAME);
        WebArea current;
        this.areas = new HashMap<String, WebArea>(areasList.getLength());
        for (int i = 0; i < areasList.getLength(); i++) {
            current = new WebAreaXML(areasList.item(i));
            this.areas.put(current.getCode(), current);
        }
        for (WebAreaList list = this.template.getAreaList(); list.hasNext();) {
            current = list.next();
            if(!this.areas.containsKey(current.getCode())) {
                this.areas.put(current.getCode(), current);
            }
        }
    }
    
    /**
     * Constructor
     * @param mod the XM node representing the page
     * @param app parent application
     * @throws InvalidXMLElementException
     */
    protected WebPageXML(Node page, WebApplication app, WebModule parent)
    throws InvalidXMLElementException, WebItemNotAvailableException, WebConfigException {
        Log.getInstance(this).enter("WebPageXML");
        this.parent = parent;
        this.webApp = app;
        Element pageElement;
        String templateName;
        if (page.getNodeType() != Node.ELEMENT_NODE) {
            throw new InvalidXMLElementException("The given XML node is not an element");
        }
        pageElement = (Element) page;
        if (!pageElement.getTagName().equals(ELEMENT_NAME)) {
            throw new InvalidXMLElementException("The given Element is not a " + ELEMENT_NAME
                                               + " element");
        }
        this.code = super.getAttribute(pageElement, ATT_CODE, true);
        Log.getInstance(this).debug("Code : " + code);
        this.encoding = super.getAttribute(pageElement, ATT_ENCODING, false, DEFAULT_ENCODING);
        this.contentType = super.getAttribute(pageElement, ATT_CONTENT_TYPE, false, DEFAULT_CONTENT_TYPE);
        templateName = super.getAttribute(pageElement, ATT_TEMPLATE, false);
        // Setting template
        if (templateName.length() == 0) {
            this.template = app.getDefaultTemplate();
        } else {
            this.template = app.getTemplate(templateName);
        }
        // Checking that he do have a template
        if (this.template == null) {
            throw new InvalidXMLElementException("Cannot find template for page : " + this.code);
        }
        // Setting initilialiser
        this.setDisplayInitialiser(super.getAttribute(pageElement, ATT_CLASS, true));
        // Setting display handler
        Log.getInstance(this).debug("Attaching display handler");
        this.displayHandler = app.getWebConfig().getDisplayHandler(
                super.getAttribute(pageElement, ATT_DI, false));
        if (this.displayHandler == null) {
            Log.getInstance(this).debug("Display handler not set or not found. Getting default one");
            this.displayHandler = app.getWebConfig().getDefaultDisplayHandler();
        }
        // If still null -> BOUM
        if (this.displayHandler == null) {
            throw new InvalidXMLElementException("Cannot determine a display handler for page: " + this.code);
        }
        Log.getInstance(this).debug("Display Handler attached");
        // Loading areas
        this.loadAreas(pageElement);
        // Loading commands
        this.loadCommands(pageElement);
        // Loading ouptu pages
        this.loadTemporaryOuputPages(pageElement);
        Log.getInstance(this).exit("WebPageXML");
    }
    
    /**
     * Set the display initialiser using the given class name
     * @param className
     */
    private void setDisplayInitialiser(String className) throws WebConfigException {
        try {
            @SuppressWarnings("unused")
            DisplayInitialiser init = (DisplayInitialiser) Class.forName(className).newInstance();
        } catch (IllegalAccessException e) {
            String message = "Insufficient rights to access " + className + ". IllegalAccessException occurs";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (ClassCastException e) {
            String message = "The class " + className + " do not implement com.jujunie.service.web.CommandXML";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (ClassNotFoundException e) {
            String message = "The class " + className + " cannot be found. Please check your classpath";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (InstantiationException e) {
            String message = "The class " + className + " do not have a public empty constructor";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        }
        this.displayInitialiser = className;
    }
    
    /**
     * @see com.jujunie.service.web.WebPage#getCode()
     */
    public String getCode() {
        return this.code;
    }

    /**
     * @see com.jujunie.service.web.WebPage#getUsedTemplate()
     */
    public WebTemplate getUsedTemplate() {
        return this.template;
    }

    /**
     * @see com.jujunie.service.web.WebPage#getArea(java.lang.String)
     */
    public WebArea getArea(String areaCode) throws WebItemNotAvailableException {
        WebArea res = (WebArea) this.areas.get(areaCode);
        if (res == null) {
            throw new WebItemNotAvailableException("The area named " + areaCode + " not found");
        }
        return res;
    }

    /**
     * @see com.jujunie.service.web.WebPage#getCommand(java.lang.String)
     */
    public Command getCommand(String code) throws WebItemNotAvailableException {
        CommandXML res = (CommandXML) this.commands.get(code);
        if (res == null) {
            String message = "The command " + code + " cannot be found";
            Log.getInstance(this).error(message);
            throw new WebItemNotAvailableException(message);
        } else {
            return res;
        }
    }

    /**
     * @see com.jujunie.service.web.WebPage#getOutput(java.lang.String)
     */
    public WebPage getOutput(String code) throws WebItemNotAvailableException {
        WebPage res = (WebPage) this.outputPages.get(code);
        if (res == null) {
            String message = "The output " + code + " cannot be found";
            Log.getInstance(this).error(message);
            throw new WebItemNotAvailableException(message);
        }
        return res;
    }
    
    /**
     * Execute operations that can be executed only when the
     * complete configuration have been loaded
     */
    public void onWebConfigLoaded() throws WebItemNotAvailableException {
        //Loading ouput pages
        WebModule selectedModule;
        WebPage selectedPage;
        this.outputPages = new HashMap<String, WebPage>(this.tmpOutputPages.size());
        for (String[] conf : this.tmpOutputPages) {
            if (conf[1].length() == 0) {
                // If there is no module code defined, using the parent one
                if (conf[2].length() == 0) {
                    // If there is no page defined, using this
                    selectedPage = this;
                } else {
                    selectedPage = this.parent.getPage(conf[2]);
                }
            } else {
                selectedModule = this.parent.getApplication().getModule(conf[1]);
                if (conf[2].length() == 0) {
                    // If there is no page defined, using default
                    selectedPage = selectedModule.getDefaultPage();
                } else {
                    selectedPage = selectedModule.getPage(conf[2]);
                }
            }
            this.outputPages.put(conf[0], selectedPage);
        }
        this.tmpOutputPages = Collections.emptyList();
    }

    /**
     * @see com.jujunie.service.web.WebPage#getEncoding()
     */
    public String getEncoding() {
        return this.encoding;
    }

    /**
     * @see com.jujunie.service.web.WebPage#getContentType()
     */
    public String getContentType() {
        return this.contentType;
    }

    /**
     * @see com.jujunie.service.web.WebPage#getDisplayInitialiser()
     */
    public DisplayInitialiser getDisplayInitialiser() {
        try {
            DisplayInitialiser init = (DisplayInitialiser) Class.forName(this.displayInitialiser).newInstance();
            init.setParent(this);
            return init;
        } catch (IllegalAccessException e) {
            String message = " SHOULD NOT APPEND !!! ALREADY TESTED !"
                + "Insufficient rights to access " + this.displayInitialiser + ". IllegalAccessException occurs";
            Log.getInstance(this).error(message, e);
            throw new IllegalStateException(message);
        } catch (ClassNotFoundException e) {
            String message = " SHOULD NOT APPEND !!! ALREADY TESTED !"
                + "The class " + this.displayInitialiser + " cannot be found. Please check your classpath";
            Log.getInstance(this).error(message, e);
            throw new IllegalStateException(message);
        } catch (InstantiationException e) {
            String message = " SHOULD NOT APPEND !!! ALREADY TESTED !"
                + "The class " + this.displayInitialiser + " do not have a public empty constructor";
            Log.getInstance(this).error(message, e);
            throw new IllegalStateException(message);
        }
    }

    /**
     * @see com.jujunie.service.web.WebPage#getWebApplication()
     */
    public WebApplication getWebApplication() {
        return this.webApp;
    }

    /**
     * @see com.jujunie.service.web.WebPage#getDisplayHandler()
     */    public DisplayHandler getDisplayHandler() {
        return this.displayHandler;
    }

}
