/**
 * 
 */
package com.jujunie.service.web.html;

import java.util.HashMap;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.jujunie.service.log.Log;
import com.jujunie.service.web.WebConfigException;
import com.jujunie.service.web.WebElementXML;
import com.jujunie.service.web.WebItemNotAvailableException;
import com.jujunie.service.xml.InvalidXMLElementException;

/**
 * HTML element provider that load the HTML element using a XML configuration
 * @author julien
 */
public class HtmlElementsProviderXML extends WebElementXML implements HtmlElementsProvider {

    /** Node name that contains the HTML configuration */
    public static final String HTML_NODE_NAME = "html";
    
    /** Node name that contains a Table configuration */
    private static final String TABLE_NODE_NAME = "table";
    
    /** Node name that contains a olumn manager configuration */
    private static final String COLUMN_MANAGER_NODE_NAME = "column-manager";
    
    /** Map of Table elements */
    private HashMap tables = new HashMap();
    
    /**
     * @see com.jujunie.service.web.html.HtmlElementsProvider#getTable(java.lang.String)
     */
    public Table getTable(String tableName) throws WebItemNotAvailableException {
        Table res = (Table) tables.get(tableName);
        if (res == null) {
            throw new WebItemNotAvailableException("The Table HTML element named \"" +
                    tableName + "\" cannot be found");
        }
        return res;
    }
    
    /**
     * Creates a new HtmlElementsProviderXML
     * @param htmlElement XML html node
     * @throws InvalidXMLElementException if one XML element is invalid
     * @throws WebConfigException if a error occurs during dynamic class loading
     */
    public HtmlElementsProviderXML(Element htmlElement)
    throws InvalidXMLElementException, WebConfigException {
        this.loadTables(htmlElement);
    }
    
    private void loadTables(Element htmlElement) 
    throws InvalidXMLElementException, WebConfigException {
        NodeList list = htmlElement.getElementsByTagName(TABLE_NODE_NAME);
        NodeList cManList = null;
        Element current;
        Table t = null;
        RowManager rMan = null;
        ColumnManager cMan = null;
        String className = null;
        try {
            for (int i = 1; i <= list.getLength(); i++) {
                current = (Element) list.item(i - 1);
                className = super.getAttribute(current, ATT_ROW_MANAGER, true);
                rMan = (RowManager) Class.forName(className).newInstance();
                t = new Table(rMan, super.getAttribute(current, ATT_NAME, true));
                t.setStyleClass(super.getAttribute(current, ATT_STYLE_CLASS, false, ""));
                cManList = current.getElementsByTagName(COLUMN_MANAGER_NODE_NAME);
                for (int j = 1; j <= cManList.getLength(); j++) {
                    current = (Element) cManList.item(j - 1);
                    cMan = (ColumnManager) Class.forName(super.getAttribute(current, ATT_CLASS, true)).newInstance();
                    cMan.setColumnStyleClass(super.getAttribute(current, ATT_COLUMN_STYLE_CLASS, false, ""));
                    cMan.setLabelCode(super.getAttribute(current, ATT_LABEL_CODE, false, ""));
                    t.addColumnManager(cMan);
                }
                this.tables.put(t.getName(), t);
                Log.getInstance(this).debug("HTML Table " + t.getName() + " loaded");
            }
        } catch (IllegalAccessException e) {
            String message = "Insufficient rights to access " + className + ". IllegalAccessException occurs";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (ClassCastException e) {
            String message = "The class " + className + " is not implementing RowManager interface if it is a"
                     + " row manager, or ColumnManager sub class if it is a column manager";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (ClassNotFoundException e) {
            String message = "The class " + className + " cannot be found. Please check your classpath";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        } catch (InstantiationException e) {
            String message = "The class " + className + " do not have a public empty constructor";
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        }
    }

}
